@@ -4,14 +4,17 @@ try:
 except ImportError:
     from collections.abc import MutableMapping as DictMixin
 from bs4 import BeautifulSoup
+from contextlib import contextmanager
 from devpi_common.archive import Archive
 from devpi_common.types import cached_property
 from devpi_common.validation import normalize_name
 from devpi_server.log import threadlog
+try:
+    import fcntl
+except ImportError:
+    fcntl = None
 import json
-import os
 import py
-import shutil
 
 
 def get_unpack_path(stage, name, version):
@@ -24,25 +27,51 @@ def get_unpack_path(stage, name, version):
         stage.user.name, stage.index, normalize_name(name), version, "+doc")
 
 
+@contextmanager
+def locked_unpack_path(stage, name, version, remove_lock_file=False):
+    unpack_path = get_unpack_path(stage, name, version)
+    # we are using the hash file as a lock file
+    hash_path = unpack_path.new(ext="hash")
+    try:
+        with hash_path.open("a+", ensure=True) as hash_file:
+            if fcntl:
+                fcntl.flock(hash_file, fcntl.LOCK_EX)
+            try:
+                yield (hash_file, unpack_path)
+            finally:
+                if fcntl:
+                    fcntl.flock(hash_file, fcntl.LOCK_UN)
+    finally:
+        if remove_lock_file and hash_path.exists():
+            try:
+                hash_path.remove()
+            except py.error.ENOENT:
+                # there is a rare possibility of a race condition here
+                pass
+
+
 def unpack_docs(stage, name, version, entry):
     # unpack, maybe a bit uncarefully but in principle
     # we are not loosing the original zip file anyway
-    unpack_path = get_unpack_path(stage, name, version)
-    hash_path = unpack_path.join('.hash')
-    if hash_path.exists():
-        with hash_path.open() as f:
-            if f.read().strip() == entry.hash_spec:
-                return unpack_path
-    if unpack_path.exists():
-        unpack_path.remove()
-    with entry.file_open_read() as f:
-        with Archive(f) as archive:
-            archive.extract(unpack_path)
-    with hash_path.open('w') as f:
-        f.write(entry.hash_spec)
-    threadlog.debug("%s: unpacked %s-%s docs to %s",
-                    stage.name, name, version, unpack_path)
-    return unpack_path
+    with locked_unpack_path(stage, name, version) as (hash_file, unpack_path):
+        hash_file.seek(0)
+        if hash_file.read().strip() == entry.hash_spec:
+            return unpack_path
+        if unpack_path.exists():
+            try:
+                unpack_path.remove()
+            except py.error.ENOENT:
+                # there is a rare possibility of a race condition here
+                pass
+        with entry.file_open_read() as f:
+            with Archive(f) as archive:
+                archive.extract(unpack_path)
+        hash_file.seek(0)
+        hash_file.write(entry.hash_spec)
+        hash_file.truncate()
+        threadlog.debug("%s: unpacked %s-%s docs to %s",
+                        stage.name, name, version, unpack_path)
+        return unpack_path
 
 
 class Docs(DictMixin):
@@ -68,6 +97,8 @@ class Docs(DictMixin):
             threadlog.warn("Tried to access %s, but it doesn't exist.", self.unpack_path)
             return {}
         unpack_path = unpack_docs(self.stage, self.name, self.version, self.entry)
+        if not unpack_path.isdir():
+            return {}
         html = []
         fjson = []
         for entry in unpack_path.visit():
@@ -126,9 +157,9 @@ def remove_docs(stage, project, version):
     if stage is None:
         # the stage was removed
         return
-    directory = get_unpack_path(stage, project, version).strpath
-    if not os.path.isdir(directory):
-        threadlog.debug("ignoring lost unpacked docs: %s" % directory)
-    else:
-        threadlog.debug("removing unpacked docs: %s" % directory)
-        shutil.rmtree(directory)
+    with locked_unpack_path(stage, project, version, remove_lock_file=True) as (hash_file, directory):
+        if not directory.isdir():
+            threadlog.debug("ignoring lost unpacked docs: %s" % directory)
+        else:
+            threadlog.debug("removing unpacked docs: %s" % directory)
+            directory.remove()
