@@ -16,7 +16,14 @@ from devpi_web.main import navigation_version
 from email.utils import parsedate
 from operator import attrgetter, itemgetter
 from py.xml import html
-from pyramid.compat import decode_path_info
+try:
+    from pyramid.compat import decode_path_info
+except ImportError:
+    # pyramid is >= 2.0, which means we can assume Python 3
+    # see PEP 3333 for why we encode WSGI PATH_INFO to latin-1 before
+    # decoding it to utf-8
+    def decode_path_info(path):
+        return path.encode('latin-1').decode('utf-8')
 from pyramid.decorator import reify
 from pyramid.httpexceptions import HTTPBadGateway, HTTPError
 from pyramid.httpexceptions import HTTPFound, HTTPNotFound
@@ -110,9 +117,9 @@ def get_doc_info(context, request, version=None, check_content=True):
         if not links:
             continue
         doc_path = unpack_docs(context.stage, name, doc_version, links[0].entry)
-        if doc_path.check():
+        if doc_path.isdir():
             break
-    if doc_path is None or not doc_path.check():
+    if doc_path is None or not doc_path.isdir():
         if version == 'stable':
             raise HTTPNotFound("No stable documentation available.")
         raise HTTPNotFound("No documentation available.")
@@ -368,7 +375,7 @@ def get_docs_info(request, stage, linkstore):
         return
     name, ver = normalize_name(linkstore.project), linkstore.version
     doc_path = unpack_docs(stage, name, ver, links[0].entry)
-    if doc_path.exists():
+    if doc_path.isdir():
         return dict(
             title="%s-%s" % (name, ver),
             url=request.route_url(
