@@ -27,7 +27,9 @@ import itertools
 import os
 import py
 import shutil
+import sys
 import time
+import traceback
 
 
 hookimpl = HookimplMarker("devpiweb")
@@ -297,7 +299,10 @@ class IndexingSharedData(object):
                     project.name, at_serial=at_serial)
                 # mirrors have no docs, so we can shortcut
                 path = '/%s/%s' % (project.indexname, project.name)
-                existing = searcher.document(path=path)
+                existing = None
+                doc_num = searcher.document_number(path=path)
+                if doc_num is not None:
+                    existing = searcher.stored_fields(doc_num)
                 if existing:
                     existing_serial = existing.get('serial', -1)
                     if existing_serial >= project_serial:
@@ -366,6 +371,14 @@ class IndexingSharedData(object):
         try:
             handler(is_from_mirror, serial, indexname, names)
         except Exception:
+            (exc_type, exc_value, exc_traceback) = sys.exc_info()
+            (filename, lineno, name) = traceback.extract_tb(exc_traceback, 2)[-1][:3]
+            del exc_traceback
+            formatted_exception = ''.join(
+                traceback.format_exception_only(exc_type, exc_value)).strip()
+            log.warn(
+                "Error during indexing in %s:%s:%s %s" % (
+                    filename, lineno, name, formatted_exception))
             self.add_errored(is_from_mirror, serial, indexname, names)
         finally:
             self.queue.task_done()
@@ -573,7 +586,10 @@ class Index(object):
         data['path'] = u"/{user}/{index}/{name}".format(
             user=data['user'], index=data['index'],
             name=normalize_name(data['name']))
-        existing = searcher.document(path=data['path'])
+        existing = None
+        doc_num = searcher.document_number(path=data['path'])
+        if doc_num is not None:
+            existing = searcher.stored_fields(doc_num)
         if existing is not None:
             needs_reindex = False
             if ('+doczip' in project) != ('doc_version' in existing):
