@@ -9,7 +9,7 @@ pytestmark = pytest.mark.xfail(sys.platform.startswith("win"), run=False, reason
 
 
 def compareable_text(text):
-    return re.sub('\s+', ' ', text.strip())
+    return re.sub(r'\s+', ' ', text.strip())
 
 
 @pytest.mark.with_notifier
@@ -20,7 +20,14 @@ def test_docs_raw_view(mapp, testapp):
     mapp.upload_doc("pkg1.zip", content, "pkg1", "2.6", code=200,
                     waithooks=True)
     r = testapp.xget(302, api.index + "/pkg1/2.6/+doc/")
-    testapp.xget(200, r.location)
+    r = testapp.xget(200, r.location)
+    assert r.cache_control.no_cache is None
+    r = testapp.xget(302, api.index + "/pkg1/latest/+doc/")
+    r = testapp.xget(200, r.location)
+    assert r.cache_control.no_cache
+    r = testapp.xget(302, api.index + "/pkg1/stable/+doc/")
+    r = testapp.xget(200, r.location)
+    assert r.cache_control.no_cache
     r = testapp.xget(404, "/blubber/blubb/pkg1/2.6/+doc/index.html")
     content, = r.html.select('#content')
     assert 'The stage blubber/blubb could not be found.' in compareable_text(content.text)
@@ -52,12 +59,15 @@ def test_docs_view(mapp, testapp):
     r = testapp.xget(404, api.index + "/pkg1/2.6/+d/foo.html")
     content, = r.html.select('#content')
     assert 'File foo.html not found in documentation.' in compareable_text(content.text)
+    r = testapp.xget(200, api.index + "/pkg1/2.6/+d/index.html?foo=bar")
+    (iframe,) = r.html.select('iframe')
+    assert iframe.attrs['src'].endswith("index.html?foo=bar")
 
 
 @pytest.mark.with_notifier
 def test_docs_raw_projectname(mapp, testapp):
     api = mapp.create_and_use()
-    content = zip_dict({"index.html": "<html><body>foo</body></"})
+    content = zip_dict({"index.html": "<html><body>foo</body></html>"})
     mapp.set_versiondata({
         "name": "pkg_hello", "version": "1.0"})
     mapp.upload_doc(
@@ -83,7 +93,7 @@ def test_docs_raw_projectname(mapp, testapp):
 @pytest.mark.with_notifier
 def test_docs_show_projectname(mapp, testapp):
     api = mapp.create_and_use()
-    content = zip_dict({"index.html": "<html><body>foo</body></"})
+    content = zip_dict({"index.html": "<html><body>foo</body></html>"})
     mapp.set_versiondata({
         "name": "pkg_hello", "version": "1.0"})
     mapp.upload_doc(
@@ -106,7 +116,7 @@ def test_docs_latest(mapp, testapp):
     assert iframe.attrs['src'] == api.index + "/pkg1/latest/+doc/index.html"
     # navigation shows latest registered version
     navigation_links = r.html.select("#navigation a")
-    assert navigation_links[3].text == '2.6'
+    assert navigation_links[4].text == '2.6'
     # there is no warning
     assert r.html.select('.infonote') == []
     # and the content matches
@@ -119,7 +129,7 @@ def test_docs_latest(mapp, testapp):
     assert iframe.attrs['src'] == api.index + "/pkg1/latest/+doc/index.html"
     # navigation shows latest registered version
     navigation_links = r.html.select("#navigation a")
-    assert navigation_links[3].text == '2.7'
+    assert navigation_links[4].text == '2.7'
     # there is a warning
     assert [x.text.strip() for x in r.html.select('.infonote')] == [
         "The latest available documentation (version 2.6) isn't for the latest available package version."]
@@ -135,7 +145,7 @@ def test_docs_latest(mapp, testapp):
     assert iframe.attrs['src'] == api.index + "/pkg1/latest/+doc/index.html"
     # navigation shows latest registered version
     navigation_links = r.html.select("#navigation a")
-    assert navigation_links[3].text == '2.7'
+    assert navigation_links[4].text == '2.7'
     # there is no warning anymore
     assert r.html.select('.infonote') == []
     # and the content is from newest docs
@@ -155,7 +165,7 @@ def test_docs_stable(mapp, testapp):
     assert iframe.attrs['src'] == api.index + "/pkg1/stable/+doc/index.html"
     # navigation shows stable registered version
     navigation_links = r.html.select("#navigation a")
-    assert navigation_links[3].text == '2.6'
+    assert navigation_links[4].text == '2.6'
     # there is no warning
     assert r.html.select('.infonote') == []
     # and the content matches
@@ -168,7 +178,7 @@ def test_docs_stable(mapp, testapp):
     assert iframe.attrs['src'] == api.index + "/pkg1/stable/+doc/index.html"
     # navigation shows stable registered version
     navigation_links = r.html.select("#navigation a")
-    assert navigation_links[3].text == '2.6'
+    assert navigation_links[4].text == '2.6'
     # there is no warning
     assert r.html.select('.infonote') == []
     # and the content is also from stable docs
@@ -183,9 +193,10 @@ def test_docs_stable(mapp, testapp):
     assert iframe.attrs['src'] == api.index + "/pkg1/stable/+doc/index.html"
     # navigation shows stable registered version
     navigation_links = r.html.select("#navigation a")
-    assert navigation_links[3].text == '2.6'
-    # still no warning
-    assert r.html.select('.infonote') == []
+    assert navigation_links[4].text == '2.6'
+    # showing latest available version
+    assert [x.text.strip() for x in r.html.select('.infonote')] == [
+        "Latest documentation"]
     # and the content is also still from stable docs
     r = testapp.xget(200, iframe.attrs['src'])
     assert r.text == "<html><body>2.6</body></html>"
@@ -196,10 +207,11 @@ def test_docs_stable(mapp, testapp):
     assert iframe.attrs['src'] == api.index + "/pkg1/stable/+doc/index.html"
     # navigation shows latest registered stable version
     navigation_links = r.html.select("#navigation a")
-    assert navigation_links[3].text == '2.7'
+    assert navigation_links[4].text == '2.7'
     # there is a warning
     assert [x.text.strip() for x in r.html.select('.infonote')] == [
-        "The latest available documentation (version 2.6) isn't for the latest available package version."]
+        "The latest available documentation (version 2.6) isn't for the latest available package version.",
+        "Latest documentation"]
     # and the content is from older stable upload
     r = testapp.xget(200, iframe.attrs['src'])
     assert r.text == "<html><body>2.6</body></html>"
@@ -212,7 +224,7 @@ def test_docs_stable(mapp, testapp):
     assert iframe.attrs['src'] == api.index + "/pkg1/stable/+doc/index.html"
     # navigation shows latest registered stable version
     navigation_links = r.html.select("#navigation a")
-    assert navigation_links[3].text == '2.7'
+    assert navigation_links[4].text == '2.7'
     # no warning anymore
     assert r.html.select('.infonote') == []
     # the content is now latest stable docs
