@@ -1,10 +1,11 @@
+from devpi_web.main import hookimpl
 from test_devpi_server.conftest import make_file_url
 import pytest
 import re
 
 
 def compareable_text(text):
-    return re.sub('\s+', ' ', text.strip())
+    return re.sub(r'\s+', ' ', text.strip())
 
 
 @pytest.mark.parametrize("input, expected", [
@@ -73,7 +74,8 @@ def test_root_pypi_upstream_error(url, mapp, testapp, pypistage):
     assert r.status_code == 502
     content, = r.html.select('#content')
     text = compareable_text(content.text)
-    assert text == 'Error An error has occurred: 502 Bad Gateway 502 status on GET https://pypi.python.org/simple/someproject/'
+    assert text.startswith('Error An error has occurred: 502 Bad Gateway 502 status on GET https://')
+    assert '/simple/someproject/' in text
 
 
 def test_error_html_only(mapp, testapp, monkeypatch):
@@ -172,6 +174,7 @@ class TestStatusView:
     @pytest.fixture
     def plugin(self):
         class Plugin:
+            @hookimpl
             def devpiweb_get_status_info(self, request):
                 result = self.results.pop()
                 if isinstance(result, Exception):
@@ -184,7 +187,7 @@ class TestStatusView:
         from devpi_web.main import get_pluginmanager
         dummyrequest.registry = pyramidconfig.registry
         dummyrequest.registry['devpi_version_info'] = []
-        pm = get_pluginmanager(load_entry_points=False)
+        pm = get_pluginmanager(xom.config, load_entry_points=False)
         pm.register(plugin)
         dummyrequest.registry['devpiweb-pluginmanager'] = pm
         dummyrequest.registry['xom'] = xom
